@@ -14,7 +14,7 @@ my $RxARF = {
     'endof'        => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
 
-sub version     { return '4.0.0' }
+sub version     { return '4.0.2' }
 sub description { return 'Abuse Feedback Reporting Format' }
 sub headerlist  { return [] }
 
@@ -67,7 +67,7 @@ sub scan {
     };
 
     my $v = undef;
-    my $p = undef;
+    my $p = '';
     push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
     $rfc822head = __PACKAGE__->RFC822HEADERS;
 
@@ -187,7 +187,7 @@ sub scan {
     } continue {
         # Save the current line for the next loop
         $p = $e;
-        $e = undef;
+        $e = '';
     }
 
     return undef unless $recipients;
@@ -228,7 +228,7 @@ sub scan {
         $e->{'spec'}    = 'SMTP';
         $e->{'action'}  = 'failed';
         $e->{'reason'}  = 'feedback';
-        $e->{'command'} = 'CONN';
+        $e->{'command'} = '';
         $e->{'agent'} ||= 'FeedBack-Loop';
     }
     return { 'ds' => $dscontents, 'rfc822' => $rfc822part };
