@@ -100,7 +100,7 @@ my $StateTable = {
     '18' => { 'command' => 'DATA', 'reason' => 'filtered' },
 };
 
-sub version     { '4.0.2' }
+sub version     { '4.0.6' }
 sub description { 'Google Gmail' }
 sub smtpagent   { 'US::Google' }
 sub headerlist  { return [ 'X-Failed-Recipients' ] }
@@ -167,6 +167,7 @@ sub scan {
     my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
     my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
     my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $previousfn = '';    # (String) Previous field name
 
     my $stripedtxt = [ split( "\n", $$mbody ) ];
@@ -175,7 +176,7 @@ sub scan {
     my $statecode0 = 0;     # (Integer) The value of (state *) in the error message
 
     my $v = undef;
-    my $p = undef;
+    my $p = '';
     push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
     $rfc822head = __PACKAGE__->RFC822HEADERS;
 
@@ -204,7 +205,14 @@ sub scan {
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
+                next if $rfc822next->{ lc $previousfn };
                 $rfc822part .= $e."\n" if $previousfn =~ m/\A(?:From|To|Subject)\z/;
+
+            } else {
+                # Check the end of headers in rfc822 part
+                next unless $previousfn =~ m/\A(?:From|To|Subject)\z/;
+                next unless $e =~ m/\A\z/;
+                $rfc822next->{ lc $previousfn } = 1;
             }
 
         } else {
@@ -247,7 +255,7 @@ sub scan {
                 if( $e =~ m/Technical details of (.+) failure:/ ) {
                     # Technical details of permanent failure: 
                     # Technical details of temporary failure: 
-                    $softbounce = 1 unless $1 eq 'permanenet';
+                    $v->{'softbounce'} = $1 eq 'permanent' ? 0 : 1;
                 }
 
                 if( $e =~ m/=\z/ ) {
@@ -266,7 +274,7 @@ sub scan {
     } continue {
         # Save the current line for the next loop
         $p = $e;
-        $e = undef;
+        $e = '';
     }
 
     return undef unless $recipients;
@@ -275,7 +283,6 @@ sub scan {
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
-        $e->{'date'}    ||= $mhead->{'date'};
         $e->{'agent'}   ||= __PACKAGE__->smtpagent;
         $e->{'diagnosis'} = Sisimai::String->sweep( $e->{'diagnosis'} );
 
@@ -310,7 +317,7 @@ sub scan {
             $e->{'command'} = $StateTable->{ $statecode0 }->{'command'};
 
         } else {
-
+            # No state code
             SESSION: for my $r ( keys %$RxErr ) {
                 # Verify each regular expression of session errors
                 PATTERN: for my $rr ( @{ $RxErr->{ $r } } ) {
@@ -323,22 +330,17 @@ sub scan {
         }
 
         $e->{'status'} = Sisimai::RFC3463->getdsn( $e->{'diagnosis'} );
-        STATUS_CODE: while(1) {
-            last if length $e->{'status'};
 
-            if( $e->{'reason'} ) {
-                # Set pseudo status code
-                $softbounce = 1 if Sisimai::RFC3463->is_softbounce( $e->{'diagnosis'} );
-                my $s = $softbounce ? 't' : 'p';
-                my $r = Sisimai::RFC3463->status( $e->{'reason'}, $s, 'i' );
-                $e->{'status'} = $r if length $r;
-            }
+        if( $e->{'reason'} ) {
+            # Set pseudo status code
+            if( $e->{'status'} =~ m/\A[45][.][1-7][.][1-9]\z/ ) {
+                # Override bounce reason 
+                $e->{'reason'} = Sisimai::RFC3463->reason( $e->{'status'} );
 
-            $e->{'status'} ||= $softbounce ? '4.0.0' : '5.0.0';
-            last;
+            } 
         }
 
-        $e->{'spec'} = $e->{'reason'} eq 'mailererror' ? 'X-UNIX' : 'SMTP';
+        $e->{'spec'}   = $e->{'reason'} eq 'mailererror' ? 'X-UNIX' : 'SMTP';
         $e->{'action'} = 'failed' if $e->{'status'} =~ m/\A[45]/;
 
     } # end of for()
