@@ -324,8 +324,8 @@ my $InternalCode = {
         'mesgtoobig'    => [ '5.0.934' ],
         'systemerror'   => [ '5.0.935' ],
         'expired'       => [ '5.0.947' ],
-        'contenterr'    => [ '5.0.960' ],
-        'securityerr'   => [ '5.0.970' ],
+        'contenterror'  => [ '5.0.960' ],
+        'securityerror' => [ '5.0.970' ],
         'blocked'       => [ '5.0.971' ],
         'suspend'       => [ '5.0.990' ],
         'mailererror'   => [ '5.0.991' ],
